# floodlight analysis tool, version 0.4, nov 12 2020

# packages
library(shiny)
library(interactions)
library(data.table)
library(ggplot2)

# create functions
# convert comma separated string into numbers
comma_separated_character_to_numbers <- function(character) {
  output <- na.omit(
    as.numeric(
      unlist(
        strsplit(
          gsub(" ", "", character), ","))))
  return(output)
}
# custom ggplot theme own ggplot theme
theme_jk2 <- function(
  legend_position = "none", base_size = 20,
  axis_tick_font_size = 20, axis_title_font_size = 24,
  y_axis_title_vjust = 0.85, margin_size = 24) { 
  theme_classic(base_size = base_size) %+replace% 
    theme(
      plot.title = element_text(hjust = 0.5),
      legend.position = legend_position,
      axis.title.x = element_text(margin = margin(t = margin_size)),
      axis.title.y = element_text(
        angle = 0, vjust = y_axis_title_vjust, 
        margin = margin(r = margin_size)),
      axis.title = element_text(
        face = "bold", color = "black", size = axis_title_font_size),
      axis.text = element_text(
        face = "bold", color= "black", size = axis_tick_font_size),
      legend.title = element_text(
        face = "bold", color = "black", size = axis_title_font_size),
      legend.text = element_text(
        face = "bold", color= "black", size = axis_tick_font_size))
}
# interaction analysis
two_by_continuous_w_jn_points_plot <- function(
  data = NULL,
  iv_name = NULL,
  dv_name = NULL,
  mod_name = NULL,
  jn_points = NULL,
  output = "reg_lines_plot",
  jitter_x_as_percentage = 0,
  jitter_y_as_percentage = 0,
  dot_alpha = 0.5,
  dot_size = 4,
  legend_position = "none",
  range_limited_to_observed = F,
  reg_line_types = c("solid", "dashed"),
  jn_line_types = c("solid", "solid"),
  sig_area_color = "green",
  sig_area_alpha = 0.08,
  nonsig_area_color = "gray",
  nonsig_area_alpha = 0.08
) {
  # remove rows with na
  dt_1 <- na.omit(data[, c(iv_name, dv_name, mod_name), with = F])
  dt_1[, paste0(iv_name, "_factor") := as.factor(get(iv_name))]
  formula_1 <- as.formula(paste0(
    dv_name, " ~ ", iv_name, " * ", mod_name))
  dt_2 <- copy(dt_1)
  names(dt_2) <- c("iv", "dv", "mod", "iv_factor")
  # jn points
  if(is.null(jn_points)) {
    johnson_neyman_result <- johnson_neyman(
      lm(dv ~ iv * mod, data = dt_2), 
      pred = iv, 
      modx = mod)
    jn_points <- johnson_neyman_result[["bounds"]]
  }
  # plot simple effects at values of moderator
  if(output == "simple_effects_plot") {
    g1 <- johnson_neyman_result[["plot"]]
    g1 <- g1 + ggtitle(paste0(
      "Johnson-Neyman Plot - ",
      "R Package 'interactions', Jacob A. Long (2020)"))
    g1 <- g1 + xlab(mod_name)
    g1 <- g1 + ylab(paste0("Slope of ", iv_name))
    return(g1)
  }
  # jitter
  x_range <- max(dt_1[, get(mod_name)]) - min(dt_1[, get(mod_name)])
  y_range <- max(dt_1[, get(dv_name)]) - min(dt_1[, get(dv_name)])
  # plot
  g1 <- ggplot(
    data = dt_1, 
    aes(x = get(mod_name), y = get(dv_name),
        color = get(paste0(iv_name, "_factor")),
        linetype = get(paste0(iv_name, "_factor"))))
  # plot points
  g1 <- g1 + geom_point(
    size = dot_size,
    alpha = dot_alpha,
    position = position_jitter(
      w = x_range * jitter_x_as_percentage / 100,
      h = y_range * jitter_y_as_percentage / 100))
  # plot regression lines
  g1 <- g1 + geom_smooth(
    formula = y ~ x,
    method = "lm", se = F)
  g1 <- g1 + scale_linetype_manual(
    values = reg_line_types)
  # limit plotting to observed data range
  jn_line_pos <- jn_points
  if(range_limited_to_observed) {
    mod_min_observed <- min(dt_1[, get(mod_name)])
    mod_max_observed <- max(dt_1[, get(mod_name)])
    if(jn_line_pos[["Lower"]] < mod_min_observed) {
      jn_line_pos[["Lower"]] <- -Inf
    }
    if(jn_line_pos[["Higher"]] > mod_max_observed) {
      jn_line_pos[["Higher"]] <- Inf
    }
  }
  # apply the theme beforehand
  g1 <- g1 + theme_jk2(legend_position = legend_position)
  # allow labeling outside the plot area
  g1 <- g1 + coord_cartesian(clip = 'off')
  g1 <- g1 + theme(plot.margin = unit(c(30, 7, 7, 7), "pt"))
  # if only one type is entered for jn line
  if(length(jn_line_types) == 1) {
    jn_line_types <- rep(jn_line_types, length(jn_line_pos))
  }
  # add a vertical line and label for each jn point
  for(i in which(is.finite(jn_line_pos))) {
    # i <- 2
    # vertical line
    g1 <- g1 + geom_vline(
      xintercept = jn_line_pos[i], 
      linetype = jn_line_types[i], 
      size = 1)
    # label jn points
    g1 <- g1 + annotate(
      geom = "text", 
      x = jn_line_pos[i], 
      y = Inf, 
      label = round(jn_line_pos[i], 2),
      hjust = 0.5, vjust = -1,
      fontface = "bold", 
      color = "black",
      size = 6)
  }
  # shade
  sig_inside_vs_outside <- ifelse(
    attributes(johnson_neyman_result)$inside, "inside", "outside")
  # if sig area is outside
  if(sig_inside_vs_outside == "outside") {
    # sig area on the left
    g1 <- g1 + annotate(
      "rect", xmin = -Inf, xmax = jn_line_pos[["Lower"]], 
      ymin = -Inf, ymax = Inf, 
      alpha = sig_area_alpha, fill = sig_area_color)
    # nonsig area in the middle
    g1 <- g1 + annotate(
      "rect", 
      xmin = jn_line_pos[["Lower"]],
      xmax = jn_line_pos[["Higher"]], 
      ymin = -Inf, ymax = Inf, 
      alpha = nonsig_area_alpha, fill = nonsig_area_color)
    # sig area on the right
    g1 <- g1 + annotate(
      "rect", xmin = jn_line_pos[["Higher"]], xmax = Inf, 
      ymin = -Inf, ymax = Inf, 
      alpha = sig_area_alpha, fill = sig_area_color)
  }
  # if sig area is inside
  if(sig_inside_vs_outside == "inside") {
    # nonsig area on the left
    g1 <- g1 + annotate(
      "rect", xmin = -Inf, xmax = jn_line_pos[["Lower"]], 
      ymin = -Inf, ymax = Inf, 
      alpha = nonsig_area_alpha, fill = nonsig_area_color)
    # sig area in the middle
    g1 <- g1 + annotate(
      "rect", 
      xmin = jn_line_pos[["Lower"]],
      xmax = jn_line_pos[["Higher"]], 
      ymin = -Inf, ymax = Inf, 
      alpha = sig_area_alpha, fill = sig_area_color)
    # nonsig area on the right
    g1 <- g1 + annotate(
      "rect", xmin = jn_line_pos[["Higher"]], xmax = Inf, 
      ymin = -Inf, ymax = Inf, 
      alpha = nonsig_area_alpha, fill = nonsig_area_color)
  }
  g1 <- g1 + labs(color = iv_name)
  g1 <- g1 + xlab(mod_name)
  g1 <- g1 + ylab(dv_name)
  print(g1)
  return(g1)
}

# set default values
plot_type = "reg_lines_plot"
x_jitter_percent = 1
y_jitter_percent = 1
dot_alpha = 50
dot_size = 4
plot_width_default_value = 1100
plot_height_default_value = 600
reg_line_types_default_value = c("solid", "dashed")
jn_line_types_default_value = c("solid", "solid")
legend_position = "right"
shiny_list_max = 1000
sig_area_color = "green"
sig_area_alpha = 8
nonsig_area_color = "pink"
nonsig_area_alpha = 8

# launch the portable app
.libPaths("./R-Portable/App/R-Portable/library")
shiny::runApp("./shiny/", launch.browser=TRUE)